#!/usr/bin/env python
# Coolan Confidential
# Copyright (c) Coolan, Inc 2014 All Rights Reserved
#
# NOTICE:  All information contained herein is, and remains
# the property of Coolan, Inc.  The intellectual and
# technical concepts contained #herein are proprietary to
# Coolan, Inc and may be covered by U.S. and Foreign Patents,
# patents in process, and are protected by trade secret or copyright law.
# Dissemination of this information or reproduction of this material
# is strictly forbidden unless prior written permission is obtained
# from Coolan, Inc.
# This code and information are provided "as is" without warranty of any
# kind, either expressed or implied, including but not limited to the
# implied warranties of mechantability and/or fitness for a
# particular purpose.
import getopt

from datetime import timedelta
import threading
from multiprocessing import Process
import random


# coolan scripts
import signal
import coolanCleanup
import gatewayClient
import collector
import timeSeriesCollector
from common import *


gatewayProperty = 'GATEWAY_RANDOM_SLEEP'
cleanerProperty = 'CLEANER_RANDOM_SLEEP'
collectorProperty = 'COLLECTOR_RANDOM_SLEEP'

# Initial sleep seconds
sleepTimes = {gatewayProperty: 360,
              cleanerProperty: 720,
              collectorProperty: 10}

threadLock = threading.Lock()
threads = []


class GatewayClientThread(threading.Thread):
    def run(self):
        sleep_time = sleepTimes[gatewayProperty]
        print "%s : Pausing logs transfer for %s seconds. It will be started at %s." % (get_timestamp(), sleep_time, (datetime.now()+timedelta(seconds=sleep_time)).strftime("%Y-%m-%d %H:%M:%S"))
        time.sleep(sleep_time)
        gatewayClient.post_logs_to_endpoint()


class CoolanCleanUpThread(threading.Thread):
    def run(self):
        sleep_time = sleepTimes[cleanerProperty]
        print "%s : Pausing removal of old logs for %s seconds. It will be started at %s." % (get_timestamp(), sleep_time, (datetime.now()+timedelta(seconds=sleep_time)).strftime("%Y-%m-%d %H:%M:%S"))
        time.sleep(sleep_time)
        coolanCleanup.remove_old_logs()


class CoolanCollectorThread(threading.Thread):
    def run(self):
        sleep_time = sleepTimes[collectorProperty]
        print "%s : Pausing data collection for %s seconds. It will be started at %s." % (get_timestamp(), sleep_time, (datetime.now()+timedelta(seconds=sleep_time)).strftime("%Y-%m-%d %H:%M:%S"))
        time.sleep(sleep_time)
        collector.main(test=False)


class TimeSeriesCollectorThread(threading.Thread):
    def run(self):
        interval = config.get(s.SETTINGS, 'time_series_interval_seconds')
        count = int(config.get(s.SETTINGS, 'random_sleep_range_seconds')) / int(interval)
        timeSeriesCollector.main(interval, count)


def create_random_sleep_time(property_name):
    if not hasattr(s, property_name):
        random_value = random.randrange(1, int(config.get(s.SETTINGS, 'random_sleep_range_seconds')))
        add_property_to_settings(property_name, random_value)


def get_sleep_time_settings(property_name):
    if hasattr(s, property_name):
        # override the default and set the value from the settings
        value = getattr(s, property_name)
        sleepTimes[property_name] = int(value)
    else:
        # Writing the random sleep times to the setting file.
        create_random_sleep_time(property_name)


def run(test=False, post_reboot=False):
    if test:
        print "%s : Manually started CoolanRunner." % get_timestamp()
        if is_config_on('run_periodic_commands'):
            collector.main(test)
        else:
            print "%s : Running periodic commands has been turned off. Skip collecting logs." % get_timestamp()

        if is_config_on('run_time_series_commands'):
            timeSeriesCollector.main(s.DEFAULT_INTERVAL, s.DEFAULT_COUNT)
        else:
            print "%s : Running time series commands has been turned off. Skip collecting time series logs." % get_timestamp()

        if is_config_on('send_logs_to_endpoint'):
            gatewayClient.post_logs_to_endpoint()
        else:
            print "%s : Sending log to Coolan endpoint has been turned off. Skip sending logs." % get_timestamp()

        if is_config_on('clean_up_old_logs'):
            coolanCleanup.remove_old_logs()
        else:
            print "%s : Cleaning old logs has been turned off. Skip cleaning old logs." % get_timestamp()

        print "%s : Finished CoolanRunner." % get_timestamp()

    elif is_config_on('run_coolan_runner') and post_reboot:
        start_minute = datetime.now().minute
        start_second = datetime.now().second

        print "%s : Started CoolanRunner after a machine reboot." % get_timestamp()
        if is_config_on('run_periodic_commands'):
            collector.main(test)
        else:
            print "%s : Running periodic commands has been turned off. Skip collecting logs." % get_timestamp()

        if is_config_on('send_logs_to_endpoint'):
            gatewayClient.post_logs_to_endpoint()
        else:
            print "%s : Sending log to Coolan endpoint has been turned off. Skip sending logs." % get_timestamp()

        if is_config_on('clean_up_old_logs'):
            coolanCleanup.remove_old_logs()
        else:
            print "%s : Cleaning old logs has been turned off. Skip cleaning old logs." % get_timestamp()

        current_minute = datetime.now().minute
        # if the regular cronjob which runs on the top of the hour has not been kicked off yet
        if is_config_on('run_time_series_commands') and (start_minute > 0 or start_second > 0) and current_minute >= start_minute:
            interval = config.get(s.SETTINGS, 'time_series_interval_seconds')
            new_range = str((60 - current_minute) * 60)
            config.set(s.SETTINGS, 'random_sleep_range_seconds', new_range)
            count = int(config.get(s.SETTINGS, 'random_sleep_range_seconds')) / int(interval)
            timeSeriesCollector.main(interval, count)
        else:
            print "%s : Running time series commands has been turned off. Skip collecting time series logs." % get_timestamp()

        print "%s : Finished CoolanRunner." % get_timestamp()

    elif is_config_on('run_coolan_runner'):
        print "%s : Started CoolanRunner." % get_timestamp()
        if is_config_on('run_time_series_commands'):
            time_series_collector_thread = TimeSeriesCollectorThread()
            time_series_collector_thread.daemon = True
            time_series_collector_thread.start()
            threads.append(time_series_collector_thread)
        else:
            print "%s : Running time series commands has been turned off. Skip collecting time series logs." % get_timestamp()

        if is_config_on('run_periodic_commands'):
            collector_thread = CoolanCollectorThread()
            collector_thread.daemon = True
            collector_thread.start()
            threads.append(collector_thread)
        else:
            print "%s : Running periodic commands has been turned off. Skip collecting logs." % get_timestamp()

        if is_config_on('send_logs_to_endpoint'):
            gateway_thread = GatewayClientThread()
            gateway_thread.daemon = True
            gateway_thread.start()
            threads.append(gateway_thread)
        else:
            print "%s : Sending log to Coolan endpoint has been turned off. Skip sending logs." % get_timestamp()

        if is_config_on('clean_up_old_logs'):
            cleanup_thread = CoolanCleanUpThread()
            cleanup_thread.daemon = True
            cleanup_thread.start()
            threads.append(cleanup_thread)
        else:
            print "%s : Cleaning old logs has been turned off. Skip cleaning old logs." % get_timestamp()

        # Wait for all threads to complete
        for t in threads:
            timeout = int(config.get(s.SETTINGS, 'collector_time_out_seconds'))
            t.join(timeout)
            if t.isAlive():
                print "%s : CoolanRunner is taking too long to run. Terminating the CoolanRunner." % get_timestamp()
                # Ensuring subprocesses are dead on exiting Python program
                os.killpg(0, signal.SIGKILL)  # kill all processes in my group
                sys.exit(1)

        print "%s : Finished CoolanRunner." % get_timestamp()


def set_random_sleep_time():
    get_sleep_time_settings(collectorProperty)
    get_sleep_time_settings(gatewayProperty)
    get_sleep_time_settings(cleanerProperty)


def main():
    try:
        # Note: options is the string of option letters that the script wants to recognize, with
        # options that require an argument followed by a colon (':') i.e. -i fileNam
        opts, args = getopt.getopt(sys.argv[1:], "c:i:", ["dry-run", "now", "time-series-dry-run", "interval=", "count=", "post-reboot"])
    except getopt.GetoptError as err:
        # print help information and exit:
        print(err)  # will print something like "option -a not recognized"
        sys.exit(2)

    time_series_dry_run = False
    interval = s.DEFAULT_INTERVAL
    count = s.DEFAULT_COUNT

    for o, a in opts:
        if o == "--dry-run":
            collector.main(test=True)
            break
        elif o == "--now":
            run(test=True)
            break
        elif o == "--post-reboot":
            run(test=False, post_reboot=True)
            break
        elif o == "--time-series-dry-run":
            time_series_dry_run = True
        elif o in ("-i", "--interval"):
            interval = a
        elif o in ("-c", "--count"):
            count = a
        else:
            assert False, "unhandled option"

    if time_series_dry_run:
        timeSeriesCollector.main(interval, count)
    if not opts:
        set_random_sleep_time()
        run(test=False)


if __name__ == "__main__":
    main()
